package com.huawei.FunctionGraph.JavaPlugin.handlers;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.swing.filechooser.FileSystemView;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageDeclaration;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.ui.packageview.PackageFragmentRootContainer;
import org.eclipse.jdt.ui.jarpackager.IJarExportRunnable;
import org.eclipse.jdt.ui.jarpackager.JarPackageData;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.dialogs.TwoPaneElementSelector;
import org.eclipse.ui.internal.Workbench;
import org.yaml.snakeyaml.Yaml;
import org.yaml.snakeyaml.error.YAMLException;

import com.huawei.FunctionGraph.JavaPlugin.constant.FGSConstant;
import com.huawei.FunctionGraph.JavaPlugin.entity.ClassEntity;

/**
 * 将用户选中项目编译打包
 *
 */
@SuppressWarnings("restriction")
public class CompileHandler extends AbstractHandler {

	private Boolean flag = Boolean.FALSE;
	private StringBuilder errMsg;

	/**
	 * 用户所选项目主类
	 */
	private static String userSelectMmainClass;
	/**
	 * 用户所选项目导出地址
	 */
	private static String userSelectExportFilePath;

	public CompileHandler() {
	}

	@Override
	public Object execute(ExecutionEvent arg0) throws ExecutionException {
		int index;
		File exportFile;
		String exportFileLocatedFolder;
		String templateFile;
		String errDialogtitle;
		String yamlHandler = null;
		String jarFmt = "%s.jar";
		String zip = ".zip";
		List<ClassEntity> projectMainClass = null;

		// 获取用户选中项目
		IProject project = getProject();
		Display display = Display.getDefault();

		// 获取项目主类
		try {
			projectMainClass = getMainClass(project);
		} catch (CoreException e) {
			errDialogtitle = "Error";
			errMsg = new StringBuilder("Find main class error:").append(e.toString());
			showErrDialog(display, errDialogtitle, errMsg.toString());
			return null;
		}

		String projectPath = project.getLocation().toString();
		templateFile = String.format("%s/%s/%s", projectPath, FGSConstant.RESOURCES, FGSConstant.EXPORT_TEMPLATEYAML);

		// 设置默认导出地址 默认地址为桌面
		userSelectExportFilePath = String.format("%s\\%s.jar",
				FileSystemView.getFileSystemView().getHomeDirectory().getPath(), project.getName());

		// 从模板文件中获取函数执行入口
		try {
			yamlHandler = getHandlerFromYaml(templateFile);
		} catch (RuntimeException e) {
			errDialogtitle = "Warning";
			showErrDialog(display, errDialogtitle, errMsg.toString());
			return null;
		}

		if (yamlHandler == null) {
			// 获取函数执行入口失败
			errDialogtitle = "Warning";
			errMsg = new StringBuilder(
					"Failed to get Handler, please check if the configuration file format is incorrect.");
			showErrDialog(display, errDialogtitle, errMsg.toString());
			return null;
		}

		// 提供对话框供用户选择主类、导出地址
		show(display, yamlHandler, projectPath, projectMainClass);

		if (flag == Boolean.FALSE) {
			// 用户点击取消按钮
			return null;
		}

		if (StringUtils.isEmpty(userSelectExportFilePath) || StringUtils.isEmpty(userSelectMmainClass)) {
			errDialogtitle = "Warning";
			errMsg = new StringBuilder("Export the project to jar package is wrong, please re-select");
			showErrDialog(display, errDialogtitle, errMsg.toString());
			return null;
		}

		// 获取导出路径所在的文件夹
		index = userSelectExportFilePath.lastIndexOf(".");

		if (index == -1) {
			errDialogtitle = "Warning";
			errMsg = new StringBuilder("Export destination must be a JAR file,not a directory");
			showErrDialog(display, errDialogtitle, errMsg.toString());
			return null;
		} else {
			flag = zip.equals(userSelectExportFilePath.substring(index, userSelectExportFilePath.length()));
			if (flag) {
				exportFile = new File(String.format(jarFmt, userSelectExportFilePath.substring(0, index)));
			} else {
				exportFile = new File(userSelectExportFilePath);
			}
		}
		exportFileLocatedFolder = exportFile.getParent();

		try {
			Package(project, exportFile, flag);
			wirteYamlFile(exportFileLocatedFolder, FGSConstant.EXPORT_TEMPLATEYAML, templateFile);
		} catch (CoreException | InterruptedException | InvocationTargetException | RuntimeException e) {
			errDialogtitle = "Error";
			errMsg = new StringBuilder("An error occurred while compiling the project:");
			errMsg.append(e.getMessage());
			showErrDialog(display, errDialogtitle, errMsg.toString());
			return null;
		}
		
		// 打包结束后 删除jar
		if (flag)exportFile.delete();

		errDialogtitle = "Ok";
		errMsg = new StringBuilder("success!");
		showErrDialog(display, errDialogtitle, errMsg.toString());
		return null;
	}

	/**
	 * 提示错误对话框
	 * 
	 * @param title
	 * @param errMessage
	 */
	private void showErrDialog(Display display, String title, String errMessage) {
		MessageDialog.openInformation(display.getActiveShell(), title, errMessage);
	}

	/**
	 * 获取项目主类
	 * 
	 * @param project
	 * @return
	 * @throws CoreException
	 */
	private List<ClassEntity> getMainClass(IProject project) throws CoreException {
		ClassEntity mainClass;
		List<ClassEntity> ClassEntitys = new ArrayList<>();
		String className;
		IMethod[] methods = new IMethod[] {};
		IType type;
		IJavaProject javaProject = JavaCore.create(project);
		IPackageFragment[] packageFragments = javaProject.getPackageFragments();
		List<ICompilationUnit> compilationUnits = new ArrayList<>(16);

		for (IPackageFragment ip : packageFragments) {
			compilationUnits.addAll(Arrays.asList(ip.getCompilationUnits()));
		}

		for (ICompilationUnit compilationUnit : compilationUnits) {
			className = compilationUnit.getElementName();
			IType[] allTypes = compilationUnit.getAllTypes();
			type = allTypes.length > 0 ? allTypes[0] : compilationUnit.getType(className);
			if (type.exists()) {
				methods = allTypes[0].getMethods();
			}
			if (methods.length > 0 && methods[0].isMainMethod()) {
				mainClass = new ClassEntity();
				mainClass.setClassName(className);
				mainClass.setType(type);
				mainClass.setFullyQualifiedName(type.getFullyQualifiedName());
				IPackageDeclaration[] packageDeclarations = compilationUnit.getPackageDeclarations();
				if (packageDeclarations.length > 0) {
					mainClass.setPackageName(packageDeclarations[0].getElementName());
				}
				ClassEntitys.add(mainClass);
			}
		}
		return ClassEntitys;
	}

	/**
	 * 获取配置文件属性
	 * 
	 * @param yamlFile
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private String getHandlerFromYaml(String yamlFile) {
		Yaml yaml = new Yaml();
		Map<?, ?> map;
		String Resources = "Resources";
		String MyFunc1 = "MyFunc1";
		String Properties = "Properties";
		String Handler = "Handler";
		try {
			map = yaml.loadAs(new FileInputStream(yamlFile), Map.class);
		} catch (FileNotFoundException ioe) {
			errMsg = new StringBuilder("Unable to get the template.yaml file, please check if the file exists");
			throw new RuntimeException(errMsg.toString());
		}
		Optional<Map> opt = Optional.ofNullable(map);
		return opt.map((obj) -> (LinkedHashMap<String, LinkedHashMap>) obj.get(Resources))
				.map((obj) -> (LinkedHashMap<String, LinkedHashMap>) obj.get(MyFunc1))
				.map((obj) -> (LinkedHashMap<String, String>) obj.get(Properties)).map(obj -> obj.get(Handler))
				.orElse(null);
	}

	/**
	 * 供用户选择项目主类 、导出地址
	 * 
	 * @param disPlay
	 * @param handler
	 *            模板文件函数执行入口
	 * @param projectPath
	 *            选中项目路径
	 * @param projectMainClass
	 *            项目主类
	 * @return
	 */
	private Boolean show(Display disPlay, String handler, String projectPath, List<ClassEntity> projectMainClass) {
		Font font;
		Text selectMainCLassText;
		Text selectExportPathText;
		Button selectMainClassButton;
		Button selectExportPathButton;
		Button finshButton;
		Button cancelButton;
		Label selectMainCLassPrefixLabel;
		Label selectExportPathPrefixLabel;
		Label selectMainClassLabel;
		Label selectExportPathLabel;
		Label separationLine;

		String shellTitle = "Export";
		String selectMainClassPrefixLabelText = "Main class:";
		String selectExportPathPrefixLabelText = "JAR file:";
		String buttonText = "Browse..";
		String selectExportPathLabelText = "Select the export destination:";
		String FinshButtonText = "Finsh";
		String CancelButtonText = "Cancel";
		String selectMainClassTitle = "Select the calss of the application entry point:";
		String selectMainClassButtonMessage = "Select the class which is the application's entry point:";
		String selectMainClassButtonTitle = "Select Main Class";

		// 禁止拉伸对话框
		final Shell shell = new Shell(disPlay, SWT.CLOSE | SWT.MIN);
		shell.setText(shellTitle);
		shell.setSize(450, 430);
		shell.setImage(disPlay.getSystemImage(SWT.ICON_WORKING));
		shell.open();

		// 系统默认字体
		font = disPlay.getSystemFont();

		selectMainClassLabel = new Label(shell, SWT.None);
		selectMainClassLabel.setText(selectMainClassTitle);
		selectMainClassLabel.setBounds(10, 30, 400, 30);

		selectMainCLassPrefixLabel = new Label(shell, SWT.None);
		selectMainCLassPrefixLabel.setText(selectMainClassPrefixLabelText);
		selectMainCLassPrefixLabel.setBounds(10, 70, 70, 30);
		selectMainCLassPrefixLabel.setFont(font);

		// 显示所选文件的文本框
		selectMainCLassText = new Text(shell, SWT.BORDER);
		selectMainCLassText.setBounds(80, 70, 250, 30);
		selectMainCLassText.setText(handler);
		userSelectMmainClass = handler;

		selectMainClassButton = new Button(shell, SWT.NONE);
		selectMainClassButton.setText(buttonText);
		selectMainClassButton.setBounds(350, 70, 70, 30);
		selectMainClassButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				// TODO 添加图片
				// 上方选择框展示项目主类类名
				ILabelProvider mainClassNameLabelProvider = new LabelProvider() {
					@Override
					public String getText(Object element) {
						if (element instanceof ClassEntity) {
							return ((ClassEntity) element).getClassName();
						}
						return super.getText(element);
					}
				};
				// 下方选择框展示项目主类包名
				ILabelProvider mainClassPackageLabelProvider = new LabelProvider() {
					@Override
					public String getText(Object element) {
						if (element instanceof ClassEntity) {
							return ((ClassEntity) element).getPackageName();
						}
						return super.getText(element);
					}
				};
				TwoPaneElementSelector tpes = new TwoPaneElementSelector(shell, mainClassNameLabelProvider,
						mainClassPackageLabelProvider);
				tpes.setElements(projectMainClass.toArray());
				tpes.setTitle(selectMainClassButtonTitle);
				tpes.setMessage(selectMainClassButtonMessage);

				if (tpes.open() == Window.OK) {
					Object[] result = tpes.getResult();
					if (result != null && result.length == 1) {
						userSelectMmainClass = ((ClassEntity) result[0]).getFullyQualifiedName();
					}
					selectMainCLassText.setText(userSelectMmainClass);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
		});

		// 设置分隔线
		separationLine = new Label(shell, SWT.SEPARATOR | SWT.HORIZONTAL);
		separationLine.setBounds(0, 150, 550, 20);

		selectExportPathLabel = new Label(shell, SWT.NONE);
		selectExportPathLabel.setText(selectExportPathLabelText);
		selectExportPathLabel.setBounds(10, 200, 200, 30);

		selectExportPathPrefixLabel = new Label(shell, SWT.None);
		selectExportPathPrefixLabel.setText(selectExportPathPrefixLabelText);
		selectExportPathPrefixLabel.setBounds(10, 240, 70, 30);
		selectExportPathPrefixLabel.setFont(font);

		// jar包导出目录文本框
		selectExportPathText = new Text(shell, SWT.BORDER);
		selectExportPathText.setBounds(80, 240, 250, 30);
		selectExportPathText.setText(userSelectExportFilePath);

		selectExportPathButton = new Button(shell, SWT.NONE);
		selectExportPathButton.setText(buttonText);
		selectExportPathButton.setBounds(350, 240, 70, 30);
		selectExportPathButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				FileDialog diaLog = new FileDialog(shell, SWT.SAVE);
				diaLog.setFilterPath(projectPath);
				diaLog.setFilterExtensions(new String[] { "*.jar;*.zip", "*.jar", "*.zip" });
				String open = diaLog.open();
				selectExportPathText.setText(open == null ? userSelectExportFilePath : open);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
		});

		// 确认取消按钮
		finshButton = new Button(shell, SWT.NONE);
		finshButton.setText(FinshButtonText);
		finshButton.setBounds(110, 330, 100, 30);
		finshButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				userSelectExportFilePath = selectExportPathText.getText();
				userSelectMmainClass = selectMainCLassText.getText();
				flag = Boolean.TRUE;
				shell.close();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
		});

		cancelButton = new Button(shell, SWT.NONE);
		cancelButton.setText(CancelButtonText);
		cancelButton.setBounds(250, 330, 100, 30);
		cancelButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				shell.close();
				return;
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
		});

		while (!shell.isDisposed()) {
			if (disPlay.readAndDispatch())
				disPlay.sleep();
		}
		return flag;
	}

	/**
	 * 导出yaml文件
	 * 
	 * @param exportDir
	 *            导出文件所在文件夹
	 * @param exportYamlFileName
	 *            导出文件名
	 * @param templateFile
	 *            模板文件
	 * @throws InvocationTargetException
	 */
	private void wirteYamlFile(String exportDirLocated, String exportYamlFileName, String templateFile) {
		Map<?, ?> yamlMap = new HashMap<>();
		Yaml yaml = new Yaml();
		File file = new File(exportDirLocated + "/" + exportYamlFileName);

		if (file.exists()) {
			doDelete(file);
		}
		try (InputStream is = new FileInputStream(templateFile);
				BufferedWriter os = new BufferedWriter(new FileWriter(file))) {
			yamlMap = yaml.loadAs(is, Map.class);
			yaml.dump(yamlMap, os);
		} catch (IOException | YAMLException e) {
			errMsg = new StringBuilder("Failed to generate ").append(exportYamlFileName).append(" file").append("\n");
			errMsg.append("Error:").append(e.toString());
			throw new RuntimeException(errMsg.toString());
		}
	}

	/**
	 * 删除文件
	 * 
	 * @param path
	 */
	private void doDelete(File path) {
		if (path.isDirectory()) {
			String[] arrays = path.list();
			if (arrays == null)
				return;
			for (String file : arrays) {
				doDelete(new File(path + "//" + file));
			}
		}
		path.delete();
	}

	/**
	 * 获取用户选中项目
	 * 
	 * @return
	 */
	private IProject getProject() {
		IProject project = null;
		ISelectionService selectionService = Workbench.getInstance().getActiveWorkbenchWindow().getSelectionService();
		ISelection selection = selectionService.getSelection();
		if (selection instanceof IStructuredSelection) {
			Object element = ((IStructuredSelection) selection).getFirstElement();

			if (element instanceof IResource) {
				project = ((IResource) element).getProject();
			} else if (element instanceof IJavaElement) {
				IJavaProject jProject = ((IJavaElement) element).getJavaProject();
				project = jProject.getProject();
			} else if (element instanceof PackageFragmentRootContainer) {
				IJavaProject jProject = ((PackageFragmentRootContainer) element).getJavaProject();
				project = jProject.getProject();
			}
		}
		return project;
	}

	/**
	 * 打包
	 * 
	 * @param project
	 * @param exportFile
	 * 
	 * @throws InvocationTargetException
	 * @throws InterruptedException
	 * @throws CoreException
	 */
	private void Package(IProject project, File exportFile, Boolean exportIsJar)
			throws CoreException, InvocationTargetException, InterruptedException {
		if (project == null && exportFile == null) {
			return;
		}
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		final IJavaProject javaProject = JavaCore.create(root.getProject(project.getName()));
		IType mainType;
		String manifestFile = "META-INF/MANIFEST.MF";

		mainType = javaProject.findType(userSelectMmainClass);
		if (mainType == null) {
			errMsg = new StringBuilder("The main class of the exported jar package is abnormal.");
			throw new RuntimeException(errMsg.toString());
		}

		JarPackageData description = new JarPackageData();
		IFile ManifestFile = project.getFile(manifestFile);
		ManifestFile.refreshLocal(1, null);
		description.setManifestLocation(ManifestFile.getFullPath());
		description.setJarLocation(new Path(exportFile.getAbsolutePath()));
		description.setExportJavaFiles(true);
		description.setExportClassFiles(true);
		description.setExportOutputFolders(true);
		description.setOverwrite(true);
		description.setManifestMainClass(mainType);
		description.setElements(getJarContents(project).toArray());

		boolean manifestAccessible = description.isManifestAccessible();
		boolean valid = description.isValid();

		if (valid && manifestAccessible) {
			IJarExportRunnable runnable = description.createJarExportRunnable(Display.getDefault().getActiveShell());
			runnable.run(new NullProgressMonitor());
		} else {
			errMsg = new StringBuilder("Export project to jar package is incorrect");
			throw new RuntimeException(errMsg.toString());
		}

		if (exportIsJar) {
			File exportZipFile = new File(userSelectExportFilePath);
			byte[] bufs = new byte[1024 * 10];
			try (FileOutputStream fos = new FileOutputStream(exportZipFile);
					ZipOutputStream zos = new ZipOutputStream(new BufferedOutputStream(fos));
					BufferedInputStream bis = new BufferedInputStream(new FileInputStream(exportFile), 1024 * 10);) {
				ZipEntry zipEntry = new ZipEntry(exportFile.getName());
				zos.putNextEntry(zipEntry);
				int read = 0;
				while ((read = bis.read(bufs, 0, 1024 * 10)) != -1) {
					zos.write(bufs, 0, read);
				}
			} catch (IOException e) {
				errMsg = new StringBuilder("Failed to get zip ").append(exportZipFile.getName()).append(" file")
						.append("\n");
				errMsg.append("Error:").append(e.toString());
				throw new RuntimeException(errMsg.toString());
			}
		}
	}

	/**
	 * 获取项目可导出jar包内容
	 * 
	 * @param proj
	 * @return
	 * @throws CoreException
	 */
	private List<IResource> getJarContents(IProject proj) throws CoreException {
		final Vector<IResource> objects = new Vector<IResource>();
		proj.accept(new IResourceVisitor() {
			public boolean visit(IResource resource) throws CoreException {
				if (".settings".equals(resource.getName()) && resource.getType() == IResource.FILE
						&& resource.getParent() != null && resource.getParent().getType() == IResource.PROJECT) {
					return false;
				}
				if ("bin".equals(resource.getName()) && resource.getType() == IResource.FOLDER
						&& resource.getParent() != null && resource.getParent().getType() == IResource.PROJECT) {
					return true;
				}
				objects.add(resource);
				return true;
			}
		});
		return objects;
	}

}
