package com.huawei.FunctionGraph.JavaPlugin.handlers;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.ui.PreferenceConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;

import com.huawei.FunctionGraph.JavaPlugin.constant.FGSConstant;


/**
 * Create a JAVA template
 * 
 * @see org.eclipse.core.commands.IHandler
 * @see org.eclipse.core.commands.AbstractHandler
 */
public class CreateJavaTemplateHandler extends Wizard implements INewWizard {

    private IWorkbench workbench;
    private WizardNewProjectCreationPage wizardPage;
    private IConfigurationElement config;
    private IProject project;

    @Override
    public void init(IWorkbench workbench, IStructuredSelection selection) {
        this.workbench = workbench;
    }

    /**
     * 新增向导
     */
    @Override
    public void addPages() {
        wizardPage = new WizardNewProjectCreationPage(FGSConstant.WINDOWSTITLE);
        wizardPage.setTitle(FGSConstant.WINDOWSTITLE);
        wizardPage.setDescription(FGSConstant.DESCRIPTION);
        addPage(wizardPage);
    }

    public CreateJavaTemplateHandler() {
        super();
    }

    @Override
    public boolean needsPreviousAndNextButtons() {
        return true;
    }

    @Override
    public boolean performFinish() {
        if (project != null) {
            return Boolean.TRUE;
        }
        final IProject projectHandle = wizardPage.getProjectHandle();
        URI projectURI = (!wizardPage.useDefaults()) ? wizardPage.getLocationURI() : null;
        IWorkspace workspace = ResourcesPlugin.getWorkspace();
        final IProjectDescription desc = workspace.newProjectDescription(projectHandle.getName());
        desc.setLocationURI(projectURI);
        WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
            protected void execute(IProgressMonitor monitor) throws CoreException {
                CreateJavaTemplateProject(desc, projectHandle, monitor);
            }
        };
        try {
            getContainer().run(true, true, op);
        } catch (InterruptedException e) {
            return Boolean.FALSE;
        } catch (InvocationTargetException e) {
            Throwable realException = e.getTargetException();
            MessageDialog.openError(getShell(), "Error", realException.getMessage());
            return Boolean.FALSE;
        }

        project = projectHandle;

        if (project == null) {
            return Boolean.FALSE;
        }

        BasicNewProjectResourceWizard.updatePerspective(config);
        BasicNewProjectResourceWizard.selectAndReveal(project, workbench.getActiveWorkbenchWindow());

        return Boolean.TRUE;
    }

    /**
     * 创建JAVA模板
     * 
     * @param javaFile
     * @throws CoreException
     */
    private void CreateJavaTemplateProject(IProjectDescription description, IProject project, IProgressMonitor monitor)
            throws CoreException {
        String packagePath;
        String MainClassPath;
        String RuntimeClas;
        String RuntimeLoggerClass;
        String Context;
        String RuntimeLoggerImplClass;
        String template;

        // 设置工程标记,即为java工程
        String[] javaNature = description.getNatureIds();
        String[] newJavaNature = new String[javaNature.length + 1];
        System.arraycopy(javaNature, 0, newJavaNature, 0, javaNature.length);
        newJavaNature[javaNature.length] = FGSConstant.natureId;
        description.setNatureIds(newJavaNature);
        project.create(description, monitor);
        project.open(IResource.BACKGROUND_REFRESH, new SubProgressMonitor(monitor, 1000));

        // 转化成java工程
        IJavaProject javaProject = JavaCore.create(project);
        try {
            // 获取默认的JRE库
            IClasspathEntry[] jreLibrary = PreferenceConstants.getDefaultJRELibrary();
            // 获取原来的build path
            IClasspathEntry[] oldClasspathEntries = javaProject.getRawClasspath();
            List<IClasspathEntry> list = new ArrayList<>();
            list.addAll(Arrays.asList(jreLibrary));
            list.addAll(Arrays.asList(oldClasspathEntries));

            int classLen = jreLibrary.length + oldClasspathEntries.length;
            IClasspathEntry[] endList = new IClasspathEntry[classLen];
            System.arraycopy(jreLibrary, 0, endList, 0, jreLibrary.length);
            System.arraycopy(oldClasspathEntries, 0, endList, jreLibrary.length, oldClasspathEntries.length);

            javaProject.setRawClasspath(endList, monitor);
        } catch (JavaModelException e) {
            IStatus status = new Status(IStatus.ERROR, "CreateJavaTemplate", IStatus.OK, e.getLocalizedMessage(), null);
            throw new CoreException(status);
        }

        project.open(new SubProgressMonitor(monitor, 1000));
        IContainer container = (IContainer) project;

        // 创建文件夹
        final IFolder resources = container.getFolder(new Path(FGSConstant.RESOURCES));
        final IFolder srcFolder = container.getFolder(new Path(FGSConstant.SRC));
        resources.create(true, true, monitor);
        srcFolder.create(true, true, monitor);

        IProjectDescription description2 = javaProject.getProject().getDescription();
        ICommand command = description2.newCommand();
        command.setBuilderName(FGSConstant.BUILDER);
        description2.setBuildSpec(new ICommand[] { command });
        description2.setNatureIds(new String[] { FGSConstant.natureId });
        javaProject.getProject().setDescription(description2, null);

        IClasspathEntry srcClasspathEntry = JavaCore.newSourceEntry(srcFolder.getFullPath());

        IClasspathEntry[] oldClasspathEntries = javaProject.readRawClasspath();

        List<IClasspathEntry> list = new ArrayList<>();
        list.addAll(Arrays.asList(oldClasspathEntries));
        list.add(srcClasspathEntry);

        IClasspathEntry temp = JavaCore.newSourceEntry(new Path("/" + project.getName()));
        if (list.contains(temp)) {
            list.remove(temp);
        }
        javaProject.setRawClasspath(list.toArray(new IClasspathEntry[list.size()]), monitor);

        packagePath = String.format(FGSConstant.PackageFmt, project.getName(), FGSConstant.SRC);

        String classPathFmt = "%s/%s/%s";
        MainClassPath = String.format(classPathFmt, srcFolder.getName(), FGSConstant.CFFPACKAGE.replace(".", "/"),
                FGSConstant.MainClass);
        RuntimeClas = String.format(classPathFmt, srcFolder.getName(), FGSConstant.SERVICESPACKAGE.replace(".", "/"),
                FGSConstant.RuntimeClass);
        RuntimeLoggerClass = String.format(classPathFmt, srcFolder.getName(),
                FGSConstant.SERVICESPACKAGE.replace(".", "/"), FGSConstant.RuntimeLoggerClass);
        Context = String.format(classPathFmt, srcFolder.getName(), FGSConstant.SERVICESPACKAGE.replace(".", "/"),
                FGSConstant.ContextClass);
        RuntimeLoggerImplClass = String.format(classPathFmt, srcFolder.getName(),
                FGSConstant.SERVICESPACKAGE.replace(".", "/"), FGSConstant.RuntimeImplClass);
        template = String.format(FGSConstant.pathFmt, resources.getName(), FGSConstant.TEMPLATE);

        InputStream javaCodeInput = null;
        // 创建包
        try {
            IPackageFragmentRoot packageFragmentRoot = javaProject.findPackageFragmentRoot(new Path(packagePath));
            packageFragmentRoot.createPackageFragment(FGSConstant.CFFPACKAGE, true, monitor);
            packageFragmentRoot.createPackageFragment(FGSConstant.SERVICESPACKAGE, true, monitor);
            javaCodeInput = this.getClass().getResourceAsStream(
                    String.format(FGSConstant.pathFmt, FGSConstant.TEMPLATESFILE, FGSConstant.MainResource));
            addFileToProject(container, new Path(MainClassPath), javaCodeInput, monitor);
            javaCodeInput = this.getClass().getResourceAsStream(
                    String.format(FGSConstant.pathFmt, FGSConstant.TEMPLATESFILE, FGSConstant.RuntimeResource));
            addFileToProject(container, new Path(RuntimeClas), javaCodeInput, monitor);
            javaCodeInput = this.getClass().getResourceAsStream(
                    String.format(FGSConstant.pathFmt, FGSConstant.TEMPLATESFILE, FGSConstant.RuntimeLoggerResource));
            addFileToProject(container, new Path(RuntimeLoggerClass), javaCodeInput, monitor);
            javaCodeInput = this.getClass().getResourceAsStream(
                    String.format(FGSConstant.pathFmt, FGSConstant.TEMPLATESFILE, FGSConstant.ContextResource));
            addFileToProject(container, new Path(Context), javaCodeInput, monitor);
            javaCodeInput = this.getClass().getResourceAsStream(
                    String.format(FGSConstant.pathFmt, FGSConstant.TEMPLATESFILE, FGSConstant.RuntimeImplResource));
            addFileToProject(container, new Path(RuntimeLoggerImplClass), javaCodeInput, monitor);
            javaCodeInput = this.getClass().getResourceAsStream(
                    String.format(FGSConstant.pathFmt, FGSConstant.TEMPLATESFILE, FGSConstant.TEMPLATE));
            addFileToProject(container, new Path(template), javaCodeInput, monitor);
        } catch (JavaModelException e) {
            IStatus status = new Status(IStatus.ERROR, "CreateJavaTemplate", IStatus.OK, e.getLocalizedMessage(), null);
            throw new CoreException(status);
        } finally {
            try {
                javaCodeInput.close();
            } catch (IOException e) {
                System.out.println(e.toString());
            }
            monitor.done();
        }
    }

    /**
     * Adds a new file to the project.
     * 
     * @param container
     * @param path
     * @param contentStream
     * @param monitor
     * @throws CoreException
     */
    private void addFileToProject(IContainer container, Path path, InputStream contentStream, IProgressMonitor monitor)
            throws CoreException {
        final IFile file = container.getFile(path);

        if (file.exists()) {
            file.setContents(contentStream, true, true, monitor);
        } else {
            file.create(contentStream, true, monitor);
        }
    }

}
